@@ -30,6 +30,7 @@
 #endif
 
 #include <X11/Xatom.h>
+#include <glade/glade.h>
 #include "callbacks.h"
 #include "gok-scanner.h"
 #include "gok-input.h"
@@ -46,6 +47,8 @@
 #include "gok-spy.h"
 #include "gok-data.h"
 
+static gboolean input_event_received = FALSE;
+
 static GdkFilterReturn
 gok_xkb_filter (GdkXEvent *gdk_xevent, GdkEvent *event, gpointer data)
 {
@@ -63,9 +66,17 @@ gok_input_extension_filter (GdkXEvent *gdk_xevent, GdkEvent *event, gpointer dat
 {
 	XEvent *xevent = gdk_xevent;
 #ifdef HAVE_XINPUT
+	
 	if (xevent->type == gok_input_types[GOK_INPUT_TYPE_MOTION])
 	{
 		XDeviceMotionEvent *motion = (XDeviceMotionEvent *) xevent;
+#if 0
+		if (!input_event_received && motion->deviceid != gok_input_get_current ()->device->device_id)
+		{
+			GokInput *input = gok_input_find_by_device_id (motion->deviceid, TRUE);
+			gok_input_mismatch_warn (input->info);
+		}
+#endif
 		gok_main_motion_listener (motion->axes_count, motion->axis_data, 
 					  motion->device_state, (long) motion->time);
 	}
@@ -73,6 +84,13 @@ gok_input_extension_filter (GdkXEvent *gdk_xevent, GdkEvent *event, gpointer dat
 		 (xevent->type == gok_input_types[GOK_INPUT_TYPE_BUTTON_RELEASE]))
 	{
 		XDeviceButtonEvent *button = (XDeviceButtonEvent *) xevent;
+#if 0
+		if (!input_event_received && button->deviceid != gok_input_get_current ()->device->device_id)
+		{
+			GokInput *input = gok_input_find_by_device_id (button->deviceid, TRUE);
+			gok_input_mismatch_warn (input->info);
+		}
+#endif
 		gok_main_button_listener (button->button, 
 					  (xevent->type == gok_input_types[GOK_INPUT_TYPE_BUTTON_PRESS]) ? 1 : 0, 
 					  button->state, 
@@ -82,6 +100,9 @@ gok_input_extension_filter (GdkXEvent *gdk_xevent, GdkEvent *event, gpointer dat
 	{
 		return GDK_FILTER_CONTINUE;
 	}
+	input_event_received = TRUE;
+/*	gok_input_close_others (gok_input_get_current ()); */
+
 	return GDK_FILTER_REMOVE;
 #else
 	return GDK_FILTER_CONTINUE;
@@ -98,74 +119,13 @@ on_window1_destroy                     (GtkObject       *object,
 	SPI_event_quit ();
 }
 
-GdkFilterReturn
-gok_discard_focus_filter (GdkXEvent *xevent,
-			  GdkEvent  *event,
-			  gpointer  data)
-{
-  XEvent *xev = (XEvent *)xevent;	
-  GList *list, *l;
-  gint n_mapped;
-  GtkWindow *window;
-  gboolean has_focus;
-
-  if (xev->xany.type == ClientMessage &&
-      (Atom) xev->xclient.data.l[0] == gdk_x11_atom_to_xatom (
-	      gdk_atom_intern ("WM_TAKE_FOCUS", False)))
-    {
-      list = gtk_window_list_toplevels ();
-      n_mapped = 0;
-      has_focus = FALSE;
-      for (l = list; l; l = l->next)
-        {
-          if (GTK_WIDGET_MAPPED (GTK_WIDGET (l->data)))
-            {
-              n_mapped++;
-              window = GTK_WINDOW (l->data);
-              if (window->has_focus)
-                has_focus = TRUE;
-            }
-        }
-      if (!has_focus && n_mapped > 1)
-        XSetInputFocus (xev->xany.display, GDK_WINDOW_XID (GTK_WIDGET (window)->window), RevertToPointerRoot, CurrentTime);
-
-      g_list_free (list);
-      return GDK_FILTER_REMOVE;
-    }
-  else
-    {
-      return GDK_FILTER_CONTINUE;
-    }
-}
-
 void
 on_window1_realize                     (GtkWidget       *widget,
                                         gpointer         user_data)
 {
-  XWMHints wm_hints;
-  Atom wm_window_protocols[3], atom_type_dock[1], atom_window_type;
-  static gboolean initialized = FALSE;
   gboolean is_dock = (gboolean) user_data;
-  
-  if (!initialized)
-    {
-      wm_window_protocols[0] = gdk_x11_get_xatom_by_name ("WM_DELETE_WINDOW");
-      wm_window_protocols[1] = gdk_x11_get_xatom_by_name ("_NET_WM_PING");
-      wm_window_protocols[2] = gdk_x11_get_xatom_by_name ("WM_TAKE_FOCUS");
-    }
-  
-  wm_hints.flags = InputHint;
-  wm_hints.input = False;
-  
-  gdk_error_trap_push ();
+  GdkKeymap *map = gdk_keymap_get_default ();
 
-  XSetWMHints (GDK_WINDOW_XDISPLAY (widget->window),
-	       GDK_WINDOW_XWINDOW (widget->window), &wm_hints);
-  
-  XSetWMProtocols (GDK_WINDOW_XDISPLAY (widget->window),
-		   GDK_WINDOW_XWINDOW (widget->window), wm_window_protocols, 3);
-  gdk_error_trap_pop ();
-  
   if (is_dock) {
 	  gok_main_set_wm_dock (TRUE);
   }
@@ -182,16 +142,15 @@ on_window1_realize                     (GtkWidget       *widget,
   if (!gok_keyboard_xkb_select (GDK_WINDOW_XDISPLAY (widget->window)))
 	  g_warning ("Could not register for XKB events!");
 
+  /* gok_input_open_all (); */
   gok_input_init (gok_input_get_current (), gok_input_extension_filter);
 
   gdk_window_add_filter (NULL,
 			 gok_xkb_filter,
 			 NULL);
 
-  gdk_window_add_filter (widget->window,
-			 gok_discard_focus_filter,
-			 NULL);
-
+  g_signal_connect (map, "keys-changed",
+		    (GCallback) gok_spy_keymap_listener, NULL);
 }
 
 
@@ -201,7 +160,8 @@ on_window1_button_press_event          (GtkWidget       *widget,
                                         gpointer         user_data)
 {
         GokKey *pKey = (GokKey *) user_data;
-	if (event->button == 1)
+	if (event->button == 1 && 
+	    !gok_scanner_current_state_uses_core_mouse_button (1))
 	{
 	        gok_feedback_set_selected_key (pKey);
 		gok_keyboard_output_selectedkey ();
@@ -281,7 +241,7 @@ on_window1_size_allocate               (GtkWidget       *widget,
                                         GtkAllocation   *allocation,
                                         gpointer         user_data)
 {
-/*	printf ("SizeA x = %d, y = %d, width = %d, height = %d\n", allocation->x, allocation->y, allocation->width, allocation->height); */
+        gok_log ("SizeA x = %d, y = %d, width = %d, height = %d\n", allocation->x, allocation->y, allocation->width, allocation->height); 
 	gok_keyboard_on_window_resize ();
 }
 
@@ -408,7 +368,7 @@ on_radiobuttonTypeValuator_toggled (GtkToggleButton *togglebutton,
 
 
 void
-on_combo_entry16_changed               (GtkEditable     *editable,
+on_entryActionName_changed               (GtkEditable     *editable,
                                         gpointer         user_data)
 {
 	gok_page_actions_action_changed ((GtkEditable*)editable);
@@ -855,6 +815,12 @@ on_entrySoundName_changed              (GtkEditable     *editable,
 	gok_page_feedbacks_entry_soundname_changed();
 }
 
+void
+on_SpeakLabelCheckButton_toggled (GtkToggleButton *button)
+{
+	gok_page_feedbacks_check_speech_toggled (button);
+}
+
 void
 on_notebook2_switch_page               (GtkNotebook     *notebook,
                                         GtkNotebookPage *page,
@@ -941,6 +907,122 @@ on_key_averaging_button_toggled        (GtkToggleButton *togglebutton,
 	gok_page_actions_pointer_keyaverage (gtk_toggle_button_get_active (togglebutton));
 }
 
+void
+on_ValuatorSensitivityScale_value_changed (GtkRange *range,
+					   gpointer  user_data)
+{
+	gok_data_set_valuator_sensitivity (gtk_range_get_value (range));
+}
+
+void
+on_dock_checkbutton_toggled (GtkToggleButton *button, 
+			     gpointer user_data)
+{
+	GtkWidget *dockbottom, *docktop;
+	GokDockType dock_type;
+	gboolean is_dock = gtk_toggle_button_get_active (button);
+	GladeXML *xml = gok_settingsdialog_get_xml ();
+	dockbottom = glade_xml_get_widget (xml, "DockBottomRadiobutton");
+	docktop = glade_xml_get_widget (xml, "DockTopRadiobutton");
+
+	gtk_widget_set_sensitive (docktop, is_dock);
+	gtk_widget_set_sensitive (dockbottom, is_dock);
+
+	if (is_dock)
+	{
+		if (docktop &&  gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (docktop)))
+		{
+			dock_type = GOK_DOCK_TOP;
+		}
+		else
+		{
+			dock_type = GOK_DOCK_BOTTOM;
+		}
+	}
+	else
+	{
+		dock_type = GOK_DOCK_NONE;
+	}
+	gok_data_set_dock_type (dock_type);
+}
+
+void
+on_fill_checkbutton_toggled (GtkToggleButton *button, 
+			     gpointer user_data)
+{
+	gok_data_set_expand (gtk_toggle_button_get_active (button));
+}
+
+void
+on_use_gtk_theme_checkbutton_toggled (GtkToggleButton *button,
+				      gpointer user_data)
+{
+	gok_data_set_use_gtkplus_theme (gtk_toggle_button_get_active (button));
+}
+
+void
+on_dock_top_radiobutton_toggled (GtkToggleButton *button, 
+				 gpointer user_data)
+{
+	if (gtk_toggle_button_get_active (button))
+		gok_data_set_dock_type (GOK_DOCK_TOP);
+}
+
+void
+on_dock_bottom_radiobutton_toggled (GtkToggleButton *button, 
+				    gpointer user_data)
+{
+	if (gtk_toggle_button_get_active (button))
+		gok_data_set_dock_type (GOK_DOCK_BOTTOM);
+}
+
+void
+on_aux_keyboard_dir_entry_changed (GtkEntry *entry,
+				   gpointer user_data)
+{
+	gok_data_set_aux_keyboard_directory (gtk_entry_get_text (entry));
+}
+
+
+void
+on_xkb_compose_keyboard_radiobutton_toggled (GtkToggleButton *button, 
+					     gpointer user_data)
+{
+	if (gtk_toggle_button_get_active (button))
+		gok_data_set_compose_keyboard_type (GOK_COMPOSE_XKB);
+}
+
+void
+on_alpha_compose_keyboard_radiobutton_toggled (GtkToggleButton *button, 
+					       gpointer user_data)
+{
+	if (gtk_toggle_button_get_active (button))
+		gok_data_set_compose_keyboard_type (GOK_COMPOSE_ALPHA);
+}
+
+void
+on_freq_compose_keyboard_radiobutton_toggled (GtkToggleButton *button, 
+					      gpointer user_data)
+{
+	if (gtk_toggle_button_get_active (button))
+		gok_data_set_compose_keyboard_type (GOK_COMPOSE_ALPHAFREQ);
+}
+
+void
+on_file_compose_keyboard_radiobutton_toggled (GtkToggleButton *button, 
+					      gpointer user_data)
+{
+	if (gtk_toggle_button_get_active (button))
+		gok_data_set_compose_keyboard_type (GOK_COMPOSE_CUSTOM);
+}
+
+void
+on_compose_keyboard_file_entry_changed (GtkEditable *editable, 
+					gpointer user_data)
+{
+	gok_data_set_custom_compose_filename (
+		gtk_editable_get_chars (editable, 0, -1));
+}
 
 void
 on_core_pointer_button_toggled         (GtkToggleButton *togglebutton,
@@ -965,5 +1047,53 @@ on_joystick_button_toggled             (GtkToggleButton *togglebutton,
                                         gpointer         user_data)
 {
 	gok_page_actions_set_is_corepointer (
-		!gtk_toggle_button_get_active (togglebutton));
+		!gtk_toggle_button_get_active (togglebutton));}
+
+
+void
+on_checkExtraWordList_toggled          (GtkToggleButton *togglebutton,
+                                        gpointer         user_data)
+{
+	gok_page_wordcomplete_toggle_wordlist (gtk_toggle_button_get_active (togglebutton));
 }
+
+void
+on_buttonHelp_clicked                  (GtkButton       *button,
+                                        gpointer         user_data)
+{
+	on_button_help (button, user_data);
+}
+
+
+void
+on_buttonApply_clicked                   (GtkButton       *button,
+                                        gpointer         user_data)
+{
+	on_button_try (button, user_data);
+}
+
+
+void
+on_buttonRevert_clicked                (GtkButton       *button,
+                                        gpointer         user_data)
+{
+	on_button_revert (button, user_data);
+}
+
+
+void
+on_buttonCancel_clicked                (GtkButton       *button,
+                                        gpointer         user_data)
+{
+	on_button_cancel (button, user_data);
+}
+
+
+void
+on_buttonOK_clicked                    (GtkButton       *button,
+                                        gpointer         user_data)
+{
+	on_button_ok (button, user_data);
+}
+
+
