@@ -4,8 +4,7 @@
 * Main file for the mighty GOK
 *
 * Copyright 2001 - 2004 Sun Microsystems, Inc.,
-* Copyright 2001 - 2004 University Of Toronto
-*
+* Copyright 2001 - 2004 University Of Toronto*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
@@ -771,8 +770,17 @@ gok_main_open(gint argc, gchar *argv[])
 	}
 
 	if (!gok_args.is_login) {
+	        gchar *message = g_strconcat (_("You are using GOK in \'core pointer\' mode."),"\n\n",
+					      _("The device you are using to operate GOK "
+						"is also controlling the system pointer (or \'mouse pointer\'). "
+						"Conflicts with applications\' use of the pointer may interfere "
+						"with your ability to use applications or GOK.\n\n"
+						"We strongly recommend configuring your input device as an "
+						"'Extended\' input device instead; see GOK Help for more information."),
+					      NULL);
 		gok_main_check_accessibility();
-		gok_main_warn_if_corepointer_mode (_("You are using GOK in \'core pointer\' mode."), FALSE);
+		gok_main_warn (message, FALSE, FALSE);
+		g_free (message);
 	}
 	
 	if (!gok_main_check_sticky_keys (m_pWindowMain)) {
@@ -936,8 +944,22 @@ void gok_main_window_change_listener (Accessible* pAccessible)
 
 	/* reset the context menu until we receive a focus event */
 	/*gok_spy_set_context_menu_accessible (NULL, 0);*/
-	
-	if (pAccessible && Accessible_getChildCount (pAccessible) == 1
+	if (pAccessible && (Accessible_getRole (pAccessible) == SPI_ROLE_ALERT))
+	{
+	    AccessibleNode *node = g_malloc (sizeof (AccessibleNode));
+	    GokSpyUIFlags ui_flags;
+	    ui_flags.value = 0;
+	    ui_flags.data.gui = 1;
+	    node->paccessible = pAccessible;
+	    Accessible_ref (pAccessible);
+	    node->flags.data.is_ui = 1;
+	    node->link = 0;
+	    node->pname = g_strdup (_("UI Grab"));
+	    gok_spy_update_component_list (pAccessible, ui_flags);
+	    gok_keyboard_branch_gui (node, GOK_SPY_SEARCH_UI);
+	    g_free (node);
+	}
+	else if (pAccessible && Accessible_getChildCount (pAccessible) == 1
 	    && (acc = Accessible_getChildAtIndex (pAccessible, 0))
 	    && Accessible_getRole (acc) == SPI_ROLE_MENU) 
 	{
@@ -2846,20 +2868,13 @@ gok_main_help_cb (gpointer data)
 }
  
 void
-gok_main_warn_if_corepointer_mode (gchar *message_prefix, gboolean always)
+gok_main_warn (gchar *message, gboolean always, GCallback cancelfunc)
 {
 	if ((always && !_corepointer_warning) || gok_scanner_current_state_uses_corepointer ()) 
 	{
 		GtkWidget*	button;
 		gint         response_id;
 		gboolean 	returnCode = TRUE;
-		gchar *message = g_strconcat (message_prefix, "\n\n",
-			_("This means that the device you are using to operate GOK"
-			"\nis also controlling the system pointer (or \'mouse pointer\')."
-			"\nConflicts with applications\' use of the pointer may interfere"
-			"\nwith your ability to use applications or GOK.\n"
-			"\nWe strongly recommend configuring your input device as an"
-                        "\n\'Extended\' input device instead; see GOK Help for more information."), NULL);
 		
 		if (_corepointer_warning == NULL)
 		{
@@ -2869,7 +2884,6 @@ gok_main_warn_if_corepointer_mode (gchar *message_prefix, gboolean always)
 				GTK_MESSAGE_WARNING, 
 				GTK_BUTTONS_NONE,			
 				message);
-			g_free (message);
 			
 			button = gtk_button_new_from_stock(GTK_STOCK_HELP);
 			gtk_container_add (GTK_CONTAINER (GTK_DIALOG(_corepointer_warning)->action_area),
@@ -2883,6 +2897,14 @@ gok_main_warn_if_corepointer_mode (gchar *message_prefix, gboolean always)
 			g_signal_connect (G_OBJECT (button), "clicked", 
 					  G_CALLBACK (gok_settingsdialog_show), NULL);
 			
+			if (cancelfunc != NULL)
+			{
+			    button = gtk_button_new_from_stock(GTK_STOCK_OK);
+			    gtk_container_add (GTK_CONTAINER (GTK_DIALOG(_corepointer_warning)->action_area),
+					       button);
+			    g_signal_connect_swapped (G_OBJECT (button), "clicked", 
+						      cancelfunc, _corepointer_warning);
+			}
 			button = gtk_button_new_from_stock(GTK_STOCK_OK);
 			gtk_container_add (GTK_CONTAINER (GTK_DIALOG(_corepointer_warning)->action_area),
 					   button);
